from machine import Pin, ADC, TouchPad
import time
import network
import random
from umqtt.simple import MQTTClient
from machine import Pin, PWM
from time import sleep
from config import WIFI_SSID,WIFI_PASS,MQTT_BROKER,MQTT_USER,MQTT_PASS,TOPIC_PREFIX #Have config file in micropython

########################
### Subcribed Topic ####
########################
TOPIC_LIGHT = f'{TOPIC_PREFIX}/light'
TOPIC_TIME = f'{TOPIC_PREFIX}/time'
TOPIC_BUZZER2 = f'{TOPIC_PREFIX}/buzzer2'
TOPIC_TOUCH = f'{TOPIC_PREFIX}/touch'
#######################

def connect_wifi(): #Connect to wifi
    mac = ':'.join(f'{b:02X}' for b in wifi.config('mac'))
    print(f'WiFi MAC address is {mac}')
    wifi.active(True)
    print(f'Connecting to WiFi {WIFI_SSID}.')
    wifi.connect(WIFI_SSID, WIFI_PASS)
    while not wifi.isconnected():
        print('.', end='')
        time.sleep(0.5)
    print('\nWiFi connected.')

def connect_mqtt():
    subsriptions=[TOPIC_TIME,TOPIC_BUZZER2] #subcribe only when you want incoming payload,not publish output
    print(f'Connecting to MQTT broker at {MQTT_BROKER}.')
    mqtt.connect()
    mqtt.set_callback(mqtt_callback)
    for subscription in subsriptions:
        print(f"connecting to: {subscription}...",end="")
        try:
            mqtt.subscribe(subscription)
            print("subscribed")
        except:
            print(f"\nError connecting to: {subscription}\n")
            time.sleep(1)
    print('MQTT broker connected.')

def mqtt_callback(topic, payload):
    global timer_start     #put Buzzer Timer outside function
    global alarm
    if debug:
        print(f"\nRecieved Topic: {topic.decode()}\nPayload: {payload.decode()}")
        sleep(1)
    if topic.decode() == TOPIC_TIME:
        global timer_duration
        try:
            timer_duration = int(payload.decode())
            if not debug: timer_duration*=60 #convert to minutes if debug is False
            elapsed_time_seconds = (time.ticks_ms() - timer_start)
            print("Resetting duration...")
            if elapsed_time_seconds/ 1000<= reset_limit: #Reset start time if the duration is changed within time limit
                timer_start = time.ticks_ms()  # Assigning a value to timer_start
            else:
                print(f"Timer not changed, over {reset_limit} seconds has passed ({elapsed_time_seconds/1000:.2f} sec(s))")
        except ValueError:
            pass
    elif topic.decode() == TOPIC_BUZZER2:
        try:
            deactivate_buzzer()
            alarm = False
            timer_start = now
        except ValueError:
            pass

### Function (Change default settings below) ###
def activate_buzzer(): #Activate Buzzer with specified frequency and interval
    # Set the frequency and duty cycle to generate a sound
    buzzer1.freq(buzzer_frequency)
    buzzer1.duty(buzzer_duty)
    buzzer2.freq(buzzer_frequency)
    buzzer2.duty(buzzer_duty)   
    
def deactivate_buzzer(): #Deactivate Buzzer
    # Stop the PWM output to turn off the buzzer
    buzzer1.duty(0)
    buzzer2.duty(0)
    
def check_time(timer_start, timer_duration): #Check time until set period (Return True if reached)
    elapsed_time_seconds = (time.ticks_ms() - timer_start) // 1000
    if debug:
        remaining_time_seconds = timer_duration - elapsed_time_seconds
        minutes = remaining_time_seconds // 60
        seconds = remaining_time_seconds % 60
        print(f"Time remaining: {minutes} min(s)and {seconds} sec(s)")
    if elapsed_time_seconds < timer_duration:
        return False
    else:
        return True

def someone_is_sitting(touch_sensor_value):
    #Check if someone is sitting, return True of False
    if debug:print(f"[Touch Value: {touch_sensor_value}]",end=" ")
    if touch_sensor_value >=touch_sensitivity:
        return True
    else:
        return False
###########
### Pin ###
###########
Touch_Pin = 4
buzzer1_pin = 5
buzzer2_pin = 6
##########################
##############################
#  setup (LED,Wifi,Buzzer)   #
##############################
touch_pin = TouchPad(Pin(Touch_Pin))   #Pressure sensor pin
buzzer1 = PWM(Pin(buzzer1_pin))        #First Buzzer
buzzer2 = PWM(Pin(buzzer2_pin))        #Second Buzzer
deactivate_buzzer()                    #deactivate buzzer
wifi = network.WLAN(network.STA_IF)
mqtt = MQTTClient(client_id='',
                  server=MQTT_BROKER,
                  user=MQTT_USER,
                  password=MQTT_PASS)
################################# 
#####      Settings        ######
##### (Change as seen fit) ######
#################################
global timer_duration
global alarm
PWM_FREQ = 5000                 # PWM frequency (ms?)
timer_duration = 1*60          # Buzzer duration (Minute(s)) ###########################################33
reset_limit=30                  # Max sit time(sec(s)) before timer reset is False
checkup_interval = 1 *1000      # timer check frequency (second(s)*1000 = ms)
sit_reset_duration = 30          # Sit reset duration (second(s))
touch_sensitivity = touch_pin.read()+2000   # Weight Sensor Sensitivity
buzzer_duty = 1023              # Set duty cycle to 50% (512/1024)
buzzer_frequency=1000           # Set frequency to 1000 Hz
debug = 0                    # activate print info, faster duration
if debug:
    checkup_interval+=0  #lower checkup pace
    timer_duration =5     #Buzzer duration to Sec(s)
#################################
print("\nDefault settings:")
print(f"timer_duration: {timer_duration} sec(s)\ncheckup_interval : {checkup_interval} sec(s)\nSitting reset speed: {sit_reset_duration} sec(s)\nWeight Sensor Sensitivity: {touch_sensitivity}")
print(f"Debug is {"On" if debug else "Off"}")
time.sleep(0.5)

###  Don't touch this  ###
### Universal variable ###
##########################
global timer_start              #Global buzzer timer
global alarm
timer_start = time.ticks_ms()   #Time that buzzer timer starts
last_checkup=time.ticks_ms()    #Last checkup time
last_touch=time.ticks_ms()      #Last touch checkup time
last_sit=0                      #Duration that pressure sensor returns False
alarm=False
print("\n\n\n")


connect_wifi()
connect_mqtt()
############
### MAIN ###
############
while True:
    now = time.ticks_ms()
    mqtt.check_msg()
    if now-last_checkup>=checkup_interval and alarm==False:
        last_checkup=now #Default timer = now
        touch_sensor_value = touch_pin.read()
        mqtt.publish(TOPIC_TOUCH,str(touch_sensor_value))
        check_time(timer_start, timer_duration) #print timer left till buzzing
#         if debug:print(f"Last sat: {last_sit} sec(s)",end=" ") #print last sit duration
        if someone_is_sitting(touch_sensor_value):
            if debug:print("[SOMEONE IS SITTING]",end=" ")
            last_sit=0 #reset sit timer
            if check_time(timer_start, timer_duration):
                activate_buzzer() #Buzz when timer hit 0
                print("Timer expired!")
                mqtt.publish(f'{TOPIC_PREFIX}/buzzer', '1')
                alarm=True
        else: #Noone is sitting
            timer_start+=checkup_interval #Fake pause buzzing timer, It can never buzz if some is sitting.
            last_sit+=checkup_interval #Duration that noone is sitting(ms)
            if last_sit>=sit_reset_duration/1000:
                timer_start =now #Reset timer when noone is sitting for more than sit_reset_duration sec(s)