from machine import Pin, ADC
import time
import network
from config import (
    WIFI_SSID, WIFI_PASS,
    MQTT_BROKER, MQTT_USER, MQTT_PASS,
    TOPIC_PREFIX
)
from umqtt.simple import MQTTClient
from machine import Pin, I2C
from ssd1306 import SSD1306_I2C
i2c = I2C(0, sda=Pin(48), scl=Pin(47))
oled = SSD1306_I2C(128, 32, i2c)
import random


RED_GPIO = 42
YELLOW_GPIO = 41
GREEN_GPIO = 40
LDR_GPIO = 4
TOPIC_LIGHT = f'{TOPIC_PREFIX}/light'
TOPIC_OLED = f'{TOPIC_PREFIX}/answer'
TOPIC_BUZZER = f'{TOPIC_PREFIX}/buzzer'
TOPIC_BUZZER2 = f'{TOPIC_PREFIX}/buzzer2'

ans = 0
noise = 0

def connect_wifi():
    mac = ':'.join(f'{b:02X}' for b in wifi.config('mac'))
    print(f'WiFi MAC address is {mac}')
    wifi.active(True)
    print(f'Connecting to WiFi {WIFI_SSID}.')
    wifi.connect(WIFI_SSID, WIFI_PASS)
    while not wifi.isconnected():
        print('.', end='')
        time.sleep(0.5)
    print('\nWiFi connected.')




def connect_mqtt():
    print(f'Connecting to MQTT broker at {MQTT_BROKER}.')
    mqtt.connect()
    mqtt.set_callback(mqtt_callback)
    mqtt.subscribe(TOPIC_OLED)
    mqtt.subscribe(TOPIC_BUZZER)
    mqtt.subscribe(TOPIC_BUZZER2)
    print('MQTT broker connected.')




def mqtt_callback(topic, payload):
    if topic.decode() == TOPIC_OLED:
        try:
            global ans
            ans = str(int(payload))
        except ValueError:
            pass
    if topic.decode() == TOPIC_BUZZER:
        try:
            global noise
            noise = int(payload)
        except ValueError:
            pass
    '''if topic.decode() == TOPIC_BUZZER2:
        try:
            global test
            test = int(payload)
        except ValueError:
            pass'''



############
# setup
############
red = Pin(RED_GPIO, Pin.OUT)
yellow = Pin(YELLOW_GPIO, Pin.OUT)
green = Pin(GREEN_GPIO, Pin.OUT)
ldr = ADC(Pin(LDR_GPIO), atten=ADC.ATTN_11DB)
wifi = network.WLAN(network.STA_IF)
mqtt = MQTTClient(client_id='',
                  server=MQTT_BROKER,
                  user=MQTT_USER,
                  password=MQTT_PASS)
connect_wifi()
connect_mqtt()
last_publish = 0

############
# Quick math
############
def generate_question():
    operation = random.choice(['+', '-', '*', '/'])
    if operation == '+':
        num1 = random.randint(1, 100)
        num2 = random.randint(1, 100)
        answer = num1 + num2
    elif operation == '-':
        num1 = random.randint(1, 100)
        num2 = random.randint(1, num1)
        answer = num1 - num2
    elif operation == '*':
        num1 = random.randint(1, 20)
        num2 = random.randint(1, 20)
        answer = num1 * num2
    else:
        answer = random.randint(1, 10)
        num2 = random.randint(1, 10)
        num1 = answer * num2
    
    question = f"{num1} {operation} {num2} = ? "
    return question, str(answer)

test = 0
count_test = 0
red.value(0)
yellow.value(0)
green.value(0)
############
# loop
############
mqtt.publish(TOPIC_BUZZER, '0')
while True:
    # check for incoming subscribed topics
    mqtt.check_msg()
    oled.fill(0)
    oled.show()
    if noise == 1:
        test = 1
        red.value(1)
        yellow.value(1)
        green.value(1)
    now = time.ticks_ms()
    if now - last_publish >= 2000:
        if test == 1:
            for i in range(3):
                oled.fill(0)
                oled.show()
                question, answer = generate_question()
                while int(answer) != int(ans):
                    mqtt.check_msg()
                    oled.text(question,0,0)
                    oled.show()
                if i == 0:
                    green.value(0)
                elif i == 1:
                    yellow.value(0)
                else:
                    red.value(0)
                    mqtt.publish(TOPIC_BUZZER, '0')
                    mqtt.publish(TOPIC_BUZZER2, '0')
                    oled.fill(0)
                    oled.show()
                    test = 0
                    noise = 0
    
